from subprocess import call, check_call, CalledProcessError
from os import listdir, unlink
from os.path import join, isdir, islink, exists
from .util import trace

def active_uid(uid):
    cmd = ['/usr/bin/pgrep', '-U', uid]
    ret = call(cmd, stdout=open('/dev/null'))
    if ret in (0, 1):
        return ret == 0
    raise CalledProcessError(ret, cmd)

def gc_uid(spool, uid):
    """This function conservatively attempts to garbage-collect stale uid
    reservations.
    """
    reservation = join(spool, 'uid_pool', uid)
    assert not isdir(reservation) and not islink(reservation)

    # XXX: We perform several execv()'s as root based on strings derived from
    # this 'uid' parameter, which originates as a file-name in a user-writable
    # directory. Better ideas for input validation would be welcome.
    uid_num = int(uid)
    assert uid_num >= 1000 and uid_num <= 65534  # XXX: magic numbers from util/spool.py

    if active_uid(uid):
        return

    for table in ('uid_to_instance_dir', 'uid_to_home_dir'):
        row = join(spool, table, uid)
        if exists(row):
            assert isdir(row) and not islink(row)
            check_call(['/bin/rm', '-r', '-f', row])

    # XXX: without -r or -f, we leak mail spools + ?
    # XXX: Userdel is also going to try to delete our groups for us too. Grr.
    try:
        check_call(['/usr/sbin/userdel', uid])
    except CalledProcessError, e:
        if e.returncode != 6: # ignore failure to delete non-existent users.
            raise

    # So long as we unlink the reservation last, we run no risk of seeing inconsistency
    unlink(reservation)

def gc_spool(log, spool):
    uspool = join(spool, 'uid_pool')
    if exists(uspool) and isdir(uspool):
        for maybe_uid in listdir(uspool):
            try: gc_uid(spool, maybe_uid)
            except: trace()
    else:
        log("Skipping spool %s", spool)

