import os
import sys
from signal import SIGCHLD
from optparse import OptionParser

import gobject

import dbus
import dbus.service
import dbus.mainloop.glib

from rainbow import util, inject
from rainbow.util.linux.clone import clone, CLONE_NEWNS
from rainbow.gc import gc_spool

SPOOL = '/home/olpc/isolation/1'

def log(msg, *args):
    if len(args):
        print msg % args
    else:
        print msg

class Rainbow(dbus.service.Object):
    """The Rainbow security service."""
    SERVICE_NAME = 'org.laptop.security.Rainbow'
    INTERFACE_NAME = 'org.laptop.security.Rainbow'

    def __init__(self, bus_or_name):
        dbus.service.Object.__init__(self, bus_or_name, '/')

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='s')
    def ChangeActivity(self, activity_id):
        """Sets CPU throttling on the foreground and background activities as needed."""
        pass

    @dbus.service.method(INTERFACE_NAME,
                         in_signature='sa{ss}asss',
                         async_callbacks=('success_cont', 'error_cont'),
                         utf8_strings=True)
    def CreateActivity(self, log_path, env, argv, bundle_path, bundle_id,
                       success_cont, error_cont):
        # Reap zombies
        try:
            while os.waitpid(-1, os.WNOHANG) != (0,0):
                pass
        except OSError:
            pass

        try:
            child_pid = clone(CLONE_NEWNS | SIGCHLD)
            if not child_pid:
                try:
                    log_fd = os.open(log_path, os.O_WRONLY | os.O_SYNC)
                    os.dup2(log_fd, 1)
                    os.dup2(log_fd, 2)
                    ret = inject.run(log, SPOOL, env, argv, env['SUGAR_BUNDLE_PATH'], (1, 2),
                            env.get('RAINBOW_STRACE_LOG'), 500, 500, bundle_path, bundle_id,
                            env.get('RAINBOW_CONSTANT_UID'))
                except Exception, e:
                    util.trace()
                    error_cont(e)
                finally:
                    os._exit(255)

            success_cont()

        except Exception, e:
            util.trace()
            error_cont(e)

def main():
    """Start the Rainbow DBus service."""
    parser = OptionParser(version='0.1')
    parser.add_option('-d', '--daemon', action='store_true', default=False,
                      help='Really daemonize; e.g. detach from the controlling terminal.')
    opts, args = parser.parse_args()

    if opts.daemon:
        util.daemon(0, 1)
        open('/var/run/rainbow.pid', 'w').write(str(os.getpid()))
        sys.stdout = open('/var/log/rainbow', 'w', 0)
        sys.stderr = sys.stdout

    log("GC'ing spool %s", SPOOL)
    try: gc_spool(log, SPOOL)
    except: util.trace()

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    bus = dbus.SystemBus()
    name = dbus.service.BusName(Rainbow.SERVICE_NAME, bus)
    Rainbow(name)

    print 'Service running mainloop.'
    mainloop = gobject.MainLoop()
    mainloop.run()

if __name__ == '__main__':
    main()
