# Code from http://osdir.com/ml/python.ctypes/2006-12/msg00032.html

import os
import errno
import sys
from ctypes import *

if sys.platform == 'win32':
    libc = CDLL('msvcrt.dll')
    libc._errno.restype = POINTER(c_int)
    def get_errno():
        return libc._errno().contents.value
elif sys.platform in ('linux2', 'freebsd6'):
    libc = CDLL('libc.so.6')
    libc.__errno_location.restype = POINTER(c_int)
    def get_errno():
        return libc.__errno_location().contents.value
elif sys.platform == 'darwin':
    libc = CDLL('libc.dylib')
    libc.__errno.restype = POINTER(c_int)
    def get_errno():
        return libc.__error().contents.value
else:
    raise "Ack! I don't know how to get to errno for %s" % sys.platform



class CError(Exception):
    """Base class for VServer syscall errors."""

    def __init__(self, errno_=None):
        if errno_ is None:
            errno_ = get_errno()
        self.errno = errno_
        self.errsym = errno.errorcode[errno_]
        self.errstring = os.strerror(errno_)

    def __repr__(self):
        return '<%s errno=%s>'%(self.__class__.__name__, self.errno)

    def __str__(self):
        return '%s: %s'%(self.errsym, self.errstring)


def errno_errcheck(errval, exception):
    """Used with a ctypes._CFuncPtr `func', a return code indicating error, and
       an exception type exception like:
       >>> errval = getattr(func, 'errval', -1)
       >>> func.errcheck = _errcheck(errval, excpt)"""
    def _inner(result, func, args):
        if result == errval:
            errno = get_errno()
            raise exception(errno)
        return result
    return _inner


