import os
from stat import ST_MODE, ST_UID, ST_GID

class Checker(object):
    """Sanity-check file-system permissions.
       Note: This class should NOT to be used for security-critical tests.
       In particular, this class does not know the special rules that adhere to
       uid 0.
    """
    def __init__(self, path, uid, gid):
        self.path = path
        self.uid = uid
        self.gid = gid

    def positive(self, needed, stat):
        # I'm going to try to turn off bits in mode as I verify that they can
        # be satisfied.
        observation = os.stat(self.path)
        real_mode = observation[ST_MODE]
        if self.uid == observation[ST_UID]:
            needed &= ~((real_mode & 0700) >> 6)
        if self.gid == observation[ST_GID]:
            needed &= ~((real_mode & 0070) >> 3)
        needed &= ~(real_mode & 0007)
        # Make sure needed is empty and real_mode satisfies stat.
        return (needed == 0) and ((stat & real_mode) == stat)

    def negative(self, avoid, stat):
        # I'm going to try to turn on bits for things we're permitted to do,
        # then check if any of them are disallowed.
        observation = os.stat(self.path)
        real_mode = observation[ST_MODE]
        allowed = 0
        if self.uid == observation[ST_UID]:
            allowed |= (real_mode & 0700) >> 6
        if self.gid == observation[ST_GID]:
            allowed |= (real_mode & 0070) >> 3
        allowed |= (real_mode & 0007)
        # Did we avoid everything?
        return ((avoid & allowed) == 0) and (stat & real_mode == 0)
